"""
Color scheme of elements for VESTA and Jmol.

This module contains color scheme of elements for VESTA and Jmol.
"""

# ==================================================
element_color = {
    "VESTA": {
        "Ac": "cornflowerblue",
        "Ag": "silver",
        "Al": "skyblue",
        "Am": "royalblue",
        "Ar": "gainsboro",
        "As": "yellowgreen",
        "At": "dimgrey",
        "Au": "gold",
        "B": "forestgreen",
        "Ba": "lime",
        "Be": "mediumaquamarine",
        "Bh": "crimson",
        "Bi": "darkorchid",
        "Bk": "mediumslateblue",
        "Br": "saddlebrown",
        "C": "darkslategrey",
        "Ca": "steelblue",
        "Cd": "khaki",
        "Ce": "lemonchiffon",
        "Cf": "darkorchid",
        "Cl": "lime",
        "Cm": "mediumslateblue",
        "Co": "mediumblue",
        "Cr": "darkblue",
        "Cs": "indigo",
        "Cu": "royalblue",
        "Db": "crimson",
        "Dy": "turquoise",
        "Er": "springgreen",
        "Es": "darkorchid",
        "Eu": "aquamarine",
        "F": "lightsteelblue",
        "Fe": "darkgoldenrod",
        "Fm": "darkorchid",
        "Fr": "indigo",
        "Ga": "lightgreen",
        "Gd": "turquoise",
        "Ge": "lightslategrey",
        "H": "pink",
        "He": "blanchedalmond",
        "Hf": "mediumturquoise",
        "Hg": "silver",
        "Ho": "mediumspringgreen",
        "Hs": "crimson",
        "I": "darkmagenta",
        "In": "grey",
        "Ir": "teal",
        "K": "blueviolet",
        "Kr": "mistyrose",
        "La": "limegreen",
        "Li": "lightgreen",
        "Lr": "mediumvioletred",
        "Lu": "forestgreen",
        "Md": "mediumvioletred",
        "Mg": "darkorange",
        "Mn": "darkmagenta",
        "Mo": "mediumaquamarine",
        "Mt": "crimson",
        "N": "lightsteelblue",
        "Na": "gold",
        "Nb": "mediumaquamarine",
        "Nd": "gainsboro",
        "Ne": "deeppink",
        "Ni": "silver",
        "No": "mediumvioletred",
        "Np": "dodgerblue",
        "O": "red",
        "Os": "teal",
        "P": "silver",
        "Pa": "deepskyblue",
        "Pb": "dimgrey",
        "Pd": "teal",
        "Pm": "aquamarine",
        "Po": "saddlebrown",
        "Pr": "lightgoldenrodyellow",
        "Pt": "lightgrey",
        "Pu": "dodgerblue",
        "Ra": "green",
        "Rb": "darkorchid",
        "Re": "steelblue",
        "Rf": "crimson",
        "Rh": "teal",
        "Rn": "steelblue",
        "Ru": "darkcyan",
        "S": "yellow",
        "Sb": "mediumpurple",
        "Sc": "mediumorchid",
        "Se": "chartreuse",
        "Sg": "crimson",
        "Si": "royalblue",
        "Sm": "aquamarine",
        "Sn": "darkgrey",
        "Sr": "lime",
        "Ta": "cornflowerblue",
        "Tb": "turquoise",
        "Tc": "lightseagreen",
        "Te": "darkgoldenrod",
        "Th": "deepskyblue",
        "Ti": "lightskyblue",
        "Tl": "sienna",
        "Tm": "limegreen",
        "U": "dodgerblue",
        "V": "red",
        "W": "dodgerblue",
        "Xe": "steelblue",
        "Y": "paleturquoise",
        "Yb": "limegreen",
        "Zn": "grey",
        "Zr": "lime",
    },
    "Jmol": {
        "Ac": "cornflowerblue",
        "Ag": "silver",
        "Al": "darkgrey",
        "Am": "royalblue",
        "Ar": "skyblue",
        "As": "orchid",
        "At": "dimgrey",
        "Au": "gold",
        "B": "lightpink",
        "Ba": "lime",
        "Be": "greenyellow",
        "Bh": "crimson",
        "Bi": "darkorchid",
        "Bk": "mediumslateblue",
        "Br": "brown",
        "C": "grey",
        "Ca": "lime",
        "Cd": "khaki",
        "Ce": "lemonchiffon",
        "Cf": "darkorchid",
        "Cl": "limegreen",
        "Cm": "mediumslateblue",
        "Co": "lightcoral",
        "Cr": "darkgrey",
        "Cs": "indigo",
        "Cu": "peru",
        "Db": "crimson",
        "Dy": "turquoise",
        "Er": "springgreen",
        "Es": "darkorchid",
        "Eu": "aquamarine",
        "F": "yellowgreen",
        "Fe": "chocolate",
        "Fm": "darkorchid",
        "Fr": "indigo",
        "Ga": "rosybrown",
        "Gd": "turquoise",
        "Ge": "slategrey",
        "H": "white",
        "He": "lightcyan",
        "Hf": "mediumturquoise",
        "Hg": "silver",
        "Ho": "mediumspringgreen",
        "Hs": "crimson",
        "I": "darkmagenta",
        "In": "grey",
        "Ir": "teal",
        "K": "darkorchid",
        "Kr": "mediumturquoise",
        "La": "lightskyblue",
        "Li": "violet",
        "Lr": "mediumvioletred",
        "Lu": "forestgreen",
        "Md": "mediumvioletred",
        "Mg": "chartreuse",
        "Mn": "mediumpurple",
        "Mo": "mediumaquamarine",
        "Mt": "crimson",
        "N": "royalblue",
        "Na": "mediumorchid",
        "Nb": "mediumaquamarine",
        "Nd": "gainsboro",
        "Ne": "paleturquoise",
        "Ni": "limegreen",
        "No": "mediumvioletred",
        "Np": "dodgerblue",
        "O": "red",
        "Os": "teal",
        "P": "darkorange",
        "Pa": "deepskyblue",
        "Pb": "dimgrey",
        "Pd": "teal",
        "Pm": "aquamarine",
        "Po": "saddlebrown",
        "Pr": "lightgoldenrodyellow",
        "Pt": "lightgrey",
        "Pu": "dodgerblue",
        "Ra": "green",
        "Rb": "darkorchid",
        "Re": "steelblue",
        "Rf": "crimson",
        "Rh": "teal",
        "Rn": "steelblue",
        "Ru": "darkcyan",
        "S": "yellow",
        "Sb": "mediumpurple",
        "Sc": "gainsboro",
        "Se": "orange",
        "Sg": "crimson",
        "Si": "wheat",
        "Sm": "aquamarine",
        "Sn": "slategrey",
        "Sr": "lime",
        "Ta": "cornflowerblue",
        "Tb": "turquoise",
        "Tc": "lightseagreen",
        "Te": "darkgoldenrod",
        "Th": "deepskyblue",
        "Ti": "silver",
        "Tl": "sienna",
        "Tm": "limegreen",
        "U": "dodgerblue",
        "V": "darkgrey",
        "W": "dodgerblue",
        "Xe": "steelblue",
        "Y": "paleturquoise",
        "Yb": "limegreen",
        "Zn": "lightslategrey",
        "Zr": "skyblue",
    },
}
